import unittest
from fin_crawler import FinCrawler
import time

from fin_crawler.plugins.tw_stock_info import gen_params_example,gen_params,parse


class TestCase_tw_stock_info(unittest.TestCase):

    def test_gen_params_example(self):
        self.assertEqual({},gen_params_example())

    def test_gen_params(self):

        # test normal case
        params = gen_params(**{})
        self.assertEqual(params['fetch']['url_template'],'https://mopsfin.twse.com.tw/opendata/t187ap03_L.csv')

    def test_parse(self):
        test_data = '\ufeff出表日期,公司代號,公司名稱,公司簡稱,外國企業註冊地國,產業別,住址,營利事業統一編號,董事長,總經理,發言人,發言人職稱,代理發言人,總機電話,成立日期,上市日期,普通股每股面額,實收資本額,私募股數,特別股,編制財務報表類型,股票過戶機構,過戶電話,過戶地址,簽證會計師事務所,簽證會計師1,簽證會計師2,英文簡稱,英文通訊地址,傳真機號碼,電子郵件信箱,網址\r\n"1111014","1101","台灣水泥股份有限公司","台泥","－ ","01","台北市中山北路2段113號","11913502","張安平","張安平","黃健強","資深副總經理","賴家柔","(02)2531-7099","19501229","19620209","新台幣                 10.0000元","69361817420","0","200000000","1","中國信託商業銀行代理部","66365566","台北市重慶南路一段83號5樓","勤業眾信聯合會計師事務所","黃惠敏","郭政弘","TCC","No.113, Sec.2, Zhongshan N. Rd.,Taipei City 104,Taiwan (R.O.C.)","(02)2531-6529","finance@taiwancement.com","http://www.taiwancement.com"'
        test_data = test_data.encode('utf-8')
        formed_data = {
            'website':'http://www.taiwancement.com',
            'company_name':'台灣水泥股份有限公司',
            'stock_id':'1101',
            'stock_name':'台泥'
        }

        result = parse(test_data)[0]

        for col_name,value in formed_data.items():
            self.assertEqual(value,result[col_name])


import unittest

from fin_crawler.plugins.tw_institutional_investors_daily import gen_params_example,gen_params,parse

class Test_tw_stock_price(unittest.TestCase):
    def setUp(self):
        self.date = '20221012'
    def test_gen_params_example(self):
        self.assertEqual({'date':self.date},gen_params_example())

    def test_gen_params(self):

        # test normal case
        params = gen_params(**{'date':self.date})
        self.assertEqual(
            params['fetch']['url_template'],
            'https://www.twse.com.tw/fund/T86?response=json&date=**date**&selectType=ALL&_=**time_stamp**'
            )
        self.assertEqual(params['fetch']['url_params']['**date**'],self.date)
        self.assertEqual(params['parse']['kwargs']['date'],self.date)
        # test date with -
        params = gen_params(**{'date':'2022-10-12'})
        self.assertEqual(params['fetch']['url_params']['**date**'],self.date)
        self.assertEqual(params['parse']['kwargs']['date'],self.date)
        #test invalid date
        with self.assertRaises(ValueError):
            gen_params(**{'date':'2022-09-88'})


    def test_parse(self):
        test_data = {
            'stat':'OK',
            'data':[
                ['00632R',
                '元大台灣50反1   ',
                '9,475,000',
                '10,531,000',
                '-1,056,000',
                '0',
                '0',
                '0',
                '0',
                '0',
                '0',
                '33,113,360',
                '3,589,000',
                '2,837,000',
                '752,000',
                '78,438,681',
                '46,077,321',
                '32,361,360',
                '32,057,360']
            ],
            'fields':[
                "證券代號",
                "證券名稱",
                "外陸資買進股數(不含外資自營商)",
                "外陸資賣出股數(不含外資自營商)",
                "外陸資買賣超股數(不含外資自營商)",
                "外資自營商買進股數",
                "外資自營商賣出股數",
                "外資自營商買賣超股數",
                "投信買進股數",
                "投信賣出股數",
                "投信買賣超股數",
                "自營商買賣超股數",
                "自營商買進股數(自行買賣)",
                "自營商賣出股數(自行買賣)",
                "自營商買賣超股數(自行買賣)",
                "自營商買進股數(避險)",
                "自營商賣出股數(避險)",
                "自營商買賣超股數(避險)",
                "三大法人買賣超股數"
            ]
        }
        result = parse(test_data,**{'date':self.date})
        self.assertEqual(
            result[0],
            {'stock_id': '00632R',
            'stock_name': '元大台灣50反1',
            'IIFI_buy_amount_woIIFD': 9475000.0,
            'IIFI_sell_amount_woIIFD': 10531000.0,
            'IIFI_net_amount_woIIFD': -1056000.0,
            'IIFD_buy_amount': 0.0,
            'IIFD_sell_amount': 0.0,
            'IIFD_net_amount': 0.0,
            'IIIT_buy_amount': 0.0,
            'IIIT_sell_amount': 0.0,
            'IIIT_net_amount': 0.0,
            'IID_net_amount': 33113360.0,
            'IID_buy_amount_self': 3589000.0,
            'IID_sell_amount_self': 2837000.0,
            'IID_net_amount_self': 752000.0,
            'IID_buy_amount_hedging': 78438681.0,
            'IID_sell_amount_hedging': 46077321.0,
            'IID_net_amount_hedging': 32361360.0,
            'II_net_amount': 32057360.0,
            'date': '20221012'}
        )



import time
import datetime
import copy
from .utils import convert_num,convert_tw_year
from .tw_col_names import convert_table

fields = [
    "日期",
    "成交股數",
    "成交金額",
    "開盤價",
    "最高價",
    "最低價",
    "收盤價",
    "漲跌價差",
    "成交筆數"
]

num_fields = [
    "成交股數",
    "成交金額",
    "開盤價",
    "最高價",
    "最低價",
    "收盤價",
    "漲跌價差",
    "成交筆數"
]

def parse(data,**kwargs):
    """
    資料來源:
        https://www.twse.com.tw/zh/page/trading/exchange/STOCK_DAY.html
        https://www.twse.com.tw/exchangeReport/STOCK_DAY?response=json&date=20220924&stockNo=2330&_=1664007123159
    股價:
        key: data
    欄位依序為:
        日期:date
        成交股數:volume
        成交金額:trade_amount
        開盤價:open
        最高價:high
        最低價:low
        收盤價:close
        漲跌價差:spread
        成交筆數:trade_num
    資料範例第一筆(20220924-2330):
        ['111/09/01',
        '42,008,490',
        '20,696,930,527',
        '495.00',
        '495.50',
        '490.00',
        '490.50',
        '-14.50',
        '93,631'],
    
    """

    formated_data = []

    status = data.get('stat')
    if status=='OK':
        if 'data' in data:
            rows = data['data']
            col_names = data['fields']
        else:
            print(f"Can't not find key 'data' in response data ")
            return formated_data

        for row in rows:
            template = {}
            for col_name,value in zip(col_names,row):
                template[col_name]=value

            formated_row = {}
            for col_name,value in template.items():
                converted_col_name = convert_table[col_name]

                if col_name in num_fields:
                    value = convert_num(value)
                elif col_name == '日期':
                    value = convert_tw_year(value)
                
                formated_row[converted_col_name]=value
            formated_row['stock_id']=kwargs['stock_id']
            formated_data.append(formated_row)

        return formated_data
        
    else:
        print(status)
        return formated_data

def gen_params(**params):
    """
    Require variable:
        date:20220202
        stock_id:2330

    """

    parameters_template = {
        'fetch':{
            'url_template':'https://www.twse.com.tw/exchangeReport/STOCK_DAY?response=json&date=**date**&stockNo=**stock_id**&_=**time_stamp**',
            'url_params':{
                '**date**':'20040925',
                '**time_stamp**':str(int(time.time())),
                '**stock_id**':'2330'
            },
            'method':'GET',
            'response_type':'json'
        },
        'parse':{
            'parse_data':parse,
            'kwargs':{
                'date':'20040925',
                'stock_id':'2330'
            }
        }
    }

    parameters = copy.deepcopy(parameters_template)
    date_str = params['date']
    stock_id = params['stock_id']
    #format and verify string
    date_str = date_str.replace('-','')
    datetime.datetime.strptime(date_str,'%Y%m%d')
    
    parameters['fetch']['url_params']['**date**']=date_str
    parameters['fetch']['url_params']['**stock_id**']=stock_id
    parameters['parse']['kwargs']['stock_id']=stock_id
    parameters['parse']['kwargs']['date']=date_str

    return parameters

def gen_params_example():
    example_params = {'date':'20220922','stock_id':'2330'}
    print('Get monthly stock price!')
    print(f'ex:{example_params}')
    return example_params




import unittest

from fin_crawler.plugins.tw_stock_price import gen_params_example,gen_params,parse

class Test_tw_stock_price(unittest.TestCase):
    
    def test_gen_params_example(self):
        self.assertEqual({'date':'20220922','stock_id':'2330'},gen_params_example())

    def test_gen_params(self):

        # test normal case
        params = gen_params(**{'date':'20220922','stock_id':'2330'})
        self.assertEqual(params['fetch']['url_template'],'https://www.twse.com.tw/exchangeReport/STOCK_DAY?response=json&date=**date**&stockNo=**stock_id**&_=**time_stamp**')
        self.assertEqual(params['fetch']['url_params']['**date**'],'20220922')
        self.assertEqual(params['fetch']['url_params']['**stock_id**'],'2330')
        self.assertEqual(params['parse']['kwargs']['date'],'20220922')
        self.assertEqual(params['parse']['kwargs']['stock_id'],'2330')
        # test date with -
        params = gen_params(**{'date':'2022-09-22','stock_id':'2330'})
        self.assertEqual(params['fetch']['url_params']['**date**'],'20220922')
        self.assertEqual(params['parse']['kwargs']['date'],'20220922')
        #test invalid date
        with self.assertRaises(ValueError):
            gen_params(**{'date':'2022-09-88','stock_id':'2330'})


    def test_parse(self):
        test_data = {
            'stat':'OK',
            'data':[
                ['111/09/01',
                '42,008,490',
                '20,696,930,527',
                '495.00',
                '495.50',
                '490.00',
                '490.50',
                '-14.50',
                '93,631'],
            ],
            'fields':[
                "日期",
                "成交股數",
                "成交金額",
                "開盤價",
                "最高價",
                "最低價",
                "收盤價",
                "漲跌價差",
                "成交筆數"
            ]
        }
        result = parse(test_data,**{'date':'20220922','stock_id':'2330'})[0]
        self.assertEqual(
            result,
            {'date': '20220901',
            'volume': 42008490.0,
            'trade_amount': 20696930527.0,
            'open': 495.0,
            'high': 495.5,
            'low': 490.0,
            'close': 490.5,
            'spread_value': -14.5,
            'trade_num': 93631.0,
            'stock_id': '2330'}
        )



import unittest
import math
from fin_crawler.plugins.utils import convert_num,convert_tw_year

class Test_convert_num(unittest.TestCase):

    def test_comma(self):
        self.assertEqual(123456789,convert_num('123,456,789'))
        self.assertEqual(123,convert_num('123'))
    def test_negitive(self):
        self.assertEqual(-1,convert_num('-1'))
    def test_zero(self):
        self.assertEqual(0,convert_num('X0'))
    def test_nan(self):
        self.assertEqual(True,math.isnan(convert_num('--')))
    def test_error(self):
        with self.assertRaises(ValueError):
            convert_num('1ioj2o3i')
class Test_convert_tw_year(unittest.TestCase):

    def test_convert(self):
        self.assertEqual('20220202',convert_tw_year('111/02/02'))
        self.assertEqual('20210910',convert_tw_year('110/09/10'))
        with self.assertRaises(ValueError):
            convert_tw_year('123123')


import time

class FetchParam:
    def __init__(self,url_template,url_params,method,respose_type):
        self.parms = {
            'url_template':url_template,
            'url_params':url_params,
            'method':method,
            'response_type':respose_type
        }
    
    def add_time_stamp(self):
        pass


class ParseParams:
    def __init__(self,parse_function,kwargs):
        self.params = {
            'parse_function':parse_function,
            'kwargs':kwargs
        }

class Parameter:
    def __init__(self):
        pass




import unittest

from unittest.mock import MagicMock, patch

from fin_crawler.crawler import Crawler,FinCrawler
from fin_crawler.fetcher import Fetcher

class TestCrawler(unittest.TestCase):

    def test_crawler_url(self):
        parameters = {
            "fetch":{
                "url_template":"http://test.com/?date=**date**&stockNo=**stock_id**",
                'method':'GET',
                "url_params":{
                    "**date**":"20220922",
                    "**stock_id**":"2330"
                }
            },
            'parse':{
                'parse_data':{},
                'kwargs':{}
            }
        }

        crawler = Crawler(parameters)
        self.assertEqual(crawler.fetch_params['url'],'http://test.com/?date=20220922&stockNo=2330')



#     @patch('fin_crawler.crawler.Fetcher')
#     def test_fetch(self,mock_fetcher):
#         def parse_data_function(data,**kwargs):
#             return int(data['test'])+int(kwargs['value'])
#         parameters = {
#             "fetch":{
#                 "url_template":"http://test.com/?date=**date**&stockNo=**stock_id**",
#                 "url_params":{
#                     "**date**":"20220922",
#                     "**stock_id**":"2330"
#                 }
#             },
#             'parse':{
#                 'parse_data':parse_data_function,
#                 'kwargs':{'value':'234'}
#             }
#         }


#         mock_fetcher().fetch.return_value = {'test':'123'}

#         crawler = Crawler(parameters)
        
#         self.assertEqual(crawler.fetch(),357)


# class Test_FinCrawler(unittest.TestCase):
    
#     @patch('fin_crawler.plugins.test_module.gen_params')
#     def test_get(self,mock_gen_params):

#         fetch_params = {
#             "fetch":{
#                 "url_template":"http://test.com/?date=**date**&stockNo=**stock_id**",
#                 'method':'GET',
#                 "url_params":{
#                     "**date**":"20220922",
#                     "**stock_id**":"2330"
#                 }
#             },
#             'parse':{
#                 'parse_data':{},
#                 'kwargs':{}
#             }
#         }
#         mock_gen_params.return_value = fetch_params

#         result = FinCrawler.get('test_module',{})
#         fetcher = Fetcher(fetch_params)
#         self.assertEqual(result,fetch_params)


import importlib
from fin_crawler.fetcher import Fetcher


class Crawler:

    """
    Handling crawl actions
    Will generate Crawler needed parameters and send to fetch to fetch data
    
    """

    def __init__(self,parameters: dict):

        self.parameters = parameters
        self.fetch_params = self.parameters['fetch']
        self.fetch_params['url'] = self.gen_url(self.fetch_params)
        # self.fetch_params['headers'] = self.gen_headers(self.fetch_params)
        # self.fetch_params['data'] = self.gen_data(self.fetch_params)
        self.fetcher = Fetcher(self.fetch_params)
        self.parse = self.parameters['parse']['parse_data']
        self.parse_kwargs = self.parameters['parse']['kwargs']

    def gen_url(self,fetch_params):
        url = fetch_params['url_template']
        for param_name,param_value in fetch_params['url_params'].items():
            url = url.replace(param_name,param_value)
        return url
    # def gen_headers(self,fetch_params):
    #     headers = fetch_params.get('headers_template') or {}
    #     headers_params = fetch_params.get('headers_params') or {}
    #     for name,value in headers_params.items():
    #         headers.replace(name,value)
    #     return headers
    # def gen_data(self,fetch_params):
    #     return {}
    def fetch(self)->dict:
        response_data = self.fetcher.fetch()
        return self.parse(response_data,**self.parse_kwargs)




class FinCrawler:
    plugin_path = 'fin_crawler.plugins'
    @classmethod
    def get(cls,data_type,params={}):
        gen_params = getattr(
            importlib.import_module(f'{cls.plugin_path}.{data_type}'),
            'gen_params'
            )
        crawler = Crawler(gen_params(**params))
        return crawler.fetch()
    
    @classmethod
    @property
    def crawler_list(cls):
        return getattr(importlib.import_module(cls.plugin_path),'crawler_list')
    
    @classmethod
    def params_example(cls,data_type):
        gen_params_example = getattr(
            importlib.import_module(f'{cls.plugin_path}.{data_type}'),
            'gen_params_example'
            )
        return gen_params_example()
    @classmethod
    def col_names(cls,data_type):
        gen_col_names = getattr(
            importlib.import_module(f'{cls.plugin_path}.{data_type}'),
            'gen_col_names'
            )
        return gen_col_names()


import unittest

from fin_crawler.plugins.tw_stock_price_daily import gen_params_example,gen_params,parse

class Test_tw_stock_price_dialy(unittest.TestCase):
    
    def test_gen_params_example(self):
        self.assertEqual({'date':'20220920'},gen_params_example())

    def test_gen_params(self):

        # test normal case
        params = gen_params(**{'date':'20220922'})
        self.assertEqual(params['fetch']['url_template'],'https://www.twse.com.tw/exchangeReport/MI_INDEX?response=json&date=**date**&type=ALL&_=**time_stamp**')
        self.assertEqual(params['fetch']['url_params']['**date**'],'20220922')
        self.assertEqual(params['parse']['kwargs']['date'],'20220922')
        # test date with -
        params = gen_params(**{'date':'2022-09-22','stock_id':'2330'})
        self.assertEqual(params['fetch']['url_params']['**date**'],'20220922')
        self.assertEqual(params['parse']['kwargs']['date'],'20220922')
        #test invalid date
        with self.assertRaises(ValueError):
            gen_params(**{'date':'2022-09-88'})


    def test_parse(self):
        test_data = {
            'stat':'OK',
            'data9':[
                ['0050',
                '元大台灣50',
                '5,999,746',
                '7,093',
                '675,492,164',
                '112.55',
                '113.15',
                '112.25',
                '113.05',
                '<p style= color:red>+</p>',
                '1.00',
                '113.00',
                '205',
                '113.05',
                '9',
                '0.00'],
            ],
            'fields9':[
                "證券代號",
                "證券名稱",
                "成交股數",
                "成交筆數",
                "成交金額",
                "開盤價",
                "最高價",
                "最低價",
                "收盤價",
                "漲跌(+/-)",
                "漲跌價差",
                "最後揭示買價",
                "最後揭示買量",
                "最後揭示賣價",
                "最後揭示賣量",
                "本益比"
            ]
        }
        result = parse(test_data)[0]
        self.assertEqual(result['spread'],1.0)
        self.assertEqual(result['stock_id'],'0050')
        self.assertEqual(result['high'],113.15)




import unittest
from fin_crawler import FinCrawler
import time
from math import nan
class Test_tw_stock_price_daily_real(unittest.TestCase):

    def test_request(self):
        time.sleep(4)
        result_idx1 = {
            'stock_id': '0051',
            'stock_name': '元大中型100',
            'volume': 72013.0,
            'trade_num': 148.0,
            'trade_amount': 4072517.0,
            'open': 56.5,
            'high': 56.8,
            'low': 56.4,
            'close': 56.55,
            'last_buy_price': 56.55,
            'last_buy_volume': 1.0,
            'last_sell_price': 56.6,
            'last_sell_volume': 35.0,
            'PE': 0.0,
            'spread': -1,
            'date': '20210922'
            }
        result_idx500 = {
            'stock_id': '030325',
            'stock_name': '長榮麥證0C購01',
            'volume': 0.0,
            'trade_num': 0.0,
            'trade_amount': 0.0,
            'open': nan,
            'high': nan,
            'low': nan,
            'close': nan,
            'last_buy_price': 15.6,
            'last_buy_volume': 10.0,
            'last_sell_price': nan,
            'last_sell_volume': 0.0,
            'PE': 0.0,
            'spread': -1,
            'date': '20210922'
            }

        data = FinCrawler.get('tw_stock_price_daily',{'date':'20210922'})

        self.assertEqual(data[1],result_idx1)
        self.assertEqual(data[500],result_idx500)
    
    def test_request_no_result(self):
        time.sleep(4)
        data = FinCrawler.get('tw_stock_price_daily',{'date':'20221010'})

        self.assertEqual(data,[])



import unittest
from fin_crawler import FinCrawler
import time
class Test_tw_stock_price_all_real(unittest.TestCase):
    def setUp(self):
        self.date = '20221007'
    def test_request(self):
        time.sleep(4)
        result_idx10={
            'stock_id': '2915',
            'stock_name': '潤泰全',
            'IIFI_buy_amount_woIIFD': 4154000.0,
            'IIFI_sell_amount_woIIFD': 781000.0,
            'IIFI_net_amount_woIIFD': 3373000.0,
            'IIFD_buy_amount': 0.0,
            'IIFD_sell_amount': 0.0,
            'IIFD_net_amount': 0.0,
            'IIIT_buy_amount': 0.0,
            'IIIT_sell_amount': 1000.0,
            'IIIT_net_amount': -1000.0,
            'IID_net_amount': -26000.0,
            'IID_buy_amount_self': 70000.0,
            'IID_sell_amount_self': 108000.0,
            'IID_net_amount_self': -38000.0,
            'IID_buy_amount_hedging': 18000.0,
            'IID_sell_amount_hedging': 6000.0,
            'IID_net_amount_hedging': 12000.0,
            'II_net_amount': 3346000.0,
            'date': '20221007'
        }
        result_idx500={
            'stock_id': '067464',
            'stock_name': 'T50正2元大1C購03',
            'IIFI_buy_amount_woIIFD': 0.0,
            'IIFI_sell_amount_woIIFD': 0.0,
            'IIFI_net_amount_woIIFD': 0.0,
            'IIFD_buy_amount': 0.0,
            'IIFD_sell_amount': 0.0,
            'IIFD_net_amount': 0.0,
            'IIIT_buy_amount': 0.0,
            'IIIT_sell_amount': 0.0,
            'IIIT_net_amount': 0.0,
            'IID_net_amount': 301000.0,
            'IID_buy_amount_self': 0.0,
            'IID_sell_amount_self': 0.0,
            'IID_net_amount_self': 0.0,
            'IID_buy_amount_hedging': 301000.0,
            'IID_sell_amount_hedging': 0.0,
            'IID_net_amount_hedging': 301000.0,
            'II_net_amount': 301000.0,
            'date': '20221007'
        }
        data = FinCrawler.get('tw_institutional_investors_daily',{'date':self.date})
        self.assertEqual(data[10],result_idx10)
        self.assertEqual(data[500],result_idx500)
    
    def test_request_no_data(self):
        time.sleep(4)
        data = FinCrawler.get('tw_institutional_investors_daily',{'date':'20221010'})
        self.assertEqual(data,[])




import copy
from dataclasses import fields
from .tw_col_names import convert_table
from .utils import convert_tw_year

fields = [
    "出表日期",
    "公司代號",
    "公司名稱",
    "公司簡稱",
    "外國企業註冊地國",
    "產業別",
    "住址",
    "營利事業統一編號",
    "董事長",
    "總經理",
    "發言人",
    "發言人職稱",
    "代理發言人",
    "總機電話",
    "成立日期",
    "上市日期",
    "普通股每股面額",
    "實收資本額",
    "私募股數",
    "特別股",
    "編制財務報表類型",
    "股票過戶機構",
    "過戶電話",
    "過戶地址",
    "簽證會計師事務所",
    "簽證會計師1",
    "簽證會計師2",
    "英文簡稱",
    "英文通訊地址",
    "傳真機號碼",
    "電子郵件信箱",
    "網址"
]


def parse(data,**kwargs):
    """
    資料來源:
        https://mopsfin.twse.com.tw/opendata/t187ap03_L.csv

    欄位依序為:
        出表日期:update_dated
        公司代號:stock_id
        公司名稱:company_name
        公司簡稱:stock_name
        外國企業註冊地國:foreign_register_country
        產業別:industry_type
        住址:address
        營利事業統一編號:tax_id
        董事長:chairman
        總經理:CEO
        發言人:spokesman
        發言人職稱:spokesman_title
        代理發言人:deputy_spokesman
        總機電話:phone
        成立日期:establishment_date
        上市日期:IPO_date
        普通股每股面額:common_shares_price
        實收資本額:paid_in_capital
        私募股數:private_shares_num
        特別股:special_shares_num
        編制財務報表類型:financial_report_type
        股票過戶機構:stock_transfer_agency
        過戶電話:stcok_transfer_phone
        過戶地址:stock_transfer_address
        簽證會計師事務所:accounting_firm
        簽證會計師1:accountant_1
        簽證會計師2:accountant_2
        英文簡稱:stock_name_en
        英文通訊地址:address_en
        傳真機號碼:fax
        電子郵件信箱:email
        網址:website

    """

    formated_data = []
    

    try:
        decoded_data = data.decode('utf-8')
        rows = decoded_data.split('\r\n')
        col_names = rows.pop(0)
        col_names = col_names.replace('\ufeff','').split(',')
        
        for row in rows:
            row_data = [x.replace('"','') for x in row.split('",')]
            template = {}

            for col_name,value in zip(col_names,row_data):
                template[col_name]=value
            
            formated_row = {}
            for col_name,value in template.items():
                converted_col_name = convert_table[col_name]
                value = value.replace('"','')
                if col_name =='出表日期':
                    value = convert_tw_year(value)
                formated_row[converted_col_name] = value
            formated_data.append(formated_row)
        return formated_data
    except:
        pass
    return formated_data


def gen_params(**params):
    """
    Require variable:
        date:20220202
        stock_id:2330

    """

    parameters_template = {
        'fetch':{
            'url_template':'https://mopsfin.twse.com.tw/opendata/t187ap03_L.csv',
            'url_params':{
            },
            'method':'GET',
            'response_type':'content'
        },
        'parse':{
            'parse_data':parse,
            'kwargs':{
            }
        }
    }

    parameters = copy.deepcopy(parameters_template)

    return parameters

def gen_params_example():
    example_params = {}
    print('Get company info!')
    print(f'ex:{example_params}')
    return example_params



import unittest
from fin_crawler import FinCrawler
import time
class Test_tw_stock_price_real(unittest.TestCase):

    def test_reuqest(self):
        time.sleep(4)
        result_idx1 = {
            'date': '20220902',
            'volume': 33877655.0,
            'trade_amount': 16486942365.0,
            'open': 488.0,
            'high': 489.5,
            'low': 485.0,
            'close': 485.0,
            'spread_value': -5.5,
            'trade_num': 71175.0,
            'stock_id': '2330'
            }
        result_idx5 = {
            'date': '20220908',
            'volume': 35355467.0,
            'trade_amount': 16755699888.0,
            'open': 473.0,
            'high': 475.0,
            'low': 472.0,
            'close': 475.0,
            'spread_value': 2.5,
            'trade_num': 37943.0,
            'stock_id': '2330'
            }
        data = FinCrawler.get('tw_stock_price',{'date':'20220920','stock_id':'2330'})

        self.assertEqual(data[1],result_idx1)
        self.assertEqual(data[5],result_idx5)


import math

def convert_num(num_str):
    if len(num_str)==0:
        return math.nan
    if num_str=='--':
        return math.nan
    elif '-' in num_str:
        factor = -1
        num_str = num_str.replace('-','')
    elif 'X' in num_str:
        factor = 0
        num_str = num_str.replace('X','')
    elif '+' in num_str:
        factor = 1
        num_str = num_str.replace('+','')
    else:
        factor = 1
    return float(num_str.replace(',',''))*factor


def convert_tw_year(date_str):
    if '/' in date_str:
        tw_year,month,day = date_str.split('/')
    else:
        if len(date_str)!=7:
            raise ValueError
        tw_year,month,day = date_str[:3],date_str[3:5],date_str[5:7]
    y = int(tw_year)+1911
    return f"{y}{month}{day}"




"""
    法人英文縮寫:
        法人:institutional investor => II
        外資:foreign investors => IIFI
        自營商:dealer => IID
        投信:investment trust => IIIT
    三大法人:
        "證券代號":stock_id
        "證券名稱":stock_name
        "外陸資買進股數(不含外資自營商)":IIFI_buy_amount_woIIFD
        "外陸資賣出股數(不含外資自營商)":IIFI_sell_amount_woIIFD
        "外陸資買賣超股數(不含外資自營商)":IIFI_net_amount_woIIFD
        "外資自營商買進股數":IIFD_buy_amount
        "外資自營商賣出股數":IIFD_sell_amount
        "外資自營商買賣超股數":IIFD_net_amount
        "投信買進股數":IIIT_buy_amount
        "投信賣出股數":IIIT_sell_amount
        "投信買賣超股數":IIIT_net_amount
        "自營商買賣超股數":IID_net_amount
        "自營商買進股數(自行買賣)":IID_buy_amount_self
        "自營商賣出股數(自行買賣)":IID_sell_amount_self
        "自營商買賣超股數(自行買賣)":IID_net_amount_self
        "自營商買進股數(避險)":IID_buy_amount_hedging
        "自營商賣出股數(避險)":IID_sell_amount_hedging
        "自營商買賣超股數(避險)":IID_net_amount_hedging
        "三大法人買賣超股數":II_net_amount

    股價:
        "證券代號"
        "證券名稱"
        "成交股數"
        "成交筆數"
        "成交金額"
        "開盤價"
        "最高價"
        "最低價"
        "收盤價",
        "漲跌(+/-)"
        "漲跌價差"
        "最後揭示買價"
        "最後揭示買量"
        "最後揭示賣價"
        "最後揭示賣量"
        "本益比"

    公司資訊:
        出表日期:update_dated
        公司代號:stock_id
        公司名稱:company_name
        公司簡稱:stock_name
        外國企業註冊地國:foreign_register_country
        產業別:industry_type
        住址:address
        營利事業統一編號:tax_id
        董事長:chairman
        總經理:CEO
        發言人:spokesman
        發言人職稱:spokesman_title
        代理發言人:deputy_spokesman
        總機電話:phone
        成立日期:establishment_date
        上市日期:IPO_date
        普通股每股面額:common_shares_price
        實收資本額:paid_in_capital
        私募股數:private_shares_num
        特別股:special_shares_num
        編制財務報表類型:financial_report_type
        股票過戶機構:stock_transfer_agency
        過戶電話:stcok_transfer_phone
        過戶地址:stock_transfer_address
        簽證會計師事務所:accounting_firm
        簽證會計師1:accountant_1
        簽證會計師2:accountant_2
        英文簡稱:stock_name_en
        英文通訊地址:address_en
        傳真機號碼:fax
        電子郵件信箱:email
        網址:website
"""


convert_table = {

    # twse daily stock
    "證券代號":'stock_id',
    "證券名稱":'stock_name',
    "成交股數":'volume',
    "成交筆數":'trade_num',
    "成交金額":'trade_amount',
    "開盤價":'open',
    "最高價":'high',
    "最低價":'low',
    "收盤價":'close',
    "漲跌(+/-)":'up_down',
    "漲跌價差":'spread_value',
    "最後揭示買價":'last_buy_price',
    "最後揭示買量":'last_buy_volume',
    "最後揭示賣價":'last_sell_price',
    "最後揭示賣量":'last_sell_volume',
    "本益比":'PE',

    # custom 
    '日期': 'date',
    '漲跌':'spread',

    # 三大法人
    '外陸資買進股數(不含外資自營商)': 'IIFI_buy_amount_woIIFD',
    '外陸資賣出股數(不含外資自營商)': 'IIFI_sell_amount_woIIFD',
    '外陸資買賣超股數(不含外資自營商)': 'IIFI_net_amount_woIIFD',
    '外資自營商買進股數': 'IIFD_buy_amount',
    '外資自營商賣出股數': 'IIFD_sell_amount',
    '外資自營商買賣超股數': 'IIFD_net_amount',
    '投信買進股數': 'IIIT_buy_amount',
    '投信賣出股數': 'IIIT_sell_amount',
    '投信買賣超股數': 'IIIT_net_amount',
    '自營商買賣超股數': 'IID_net_amount',
    '自營商買進股數(自行買賣)': 'IID_buy_amount_self',
    '自營商賣出股數(自行買賣)': 'IID_sell_amount_self',
    '自營商買賣超股數(自行買賣)': 'IID_net_amount_self',
    '自營商買進股數(避險)': 'IID_buy_amount_hedging',
    '自營商賣出股數(避險)': 'IID_sell_amount_hedging',
    '自營商買賣超股數(避險)': 'IID_net_amount_hedging',
    '三大法人買賣超股數': 'II_net_amount',

    #公司資訊
    '公司代號':'stock_id',
    '公司簡稱':'stock_name',
    '出表日期': 'update_date',
    '公司名稱': 'company_name',
    '外國企業註冊地國': 'foreign_register_country',
    '產業別': 'industry_type',
    '住址': 'address',
    '營利事業統一編號': 'tax_id',
    '董事長': 'chairman',
    '總經理': 'CEO',
    '發言人': 'spokesman',
    '發言人職稱': 'spokesman_title',
    '代理發言人': 'deputy_spokesman',
    '總機電話': 'phone',
    '成立日期': 'establishment_date',
    '上市日期': 'IPO_date',
    '普通股每股面額': 'common_shares_price',
    '實收資本額': 'paid_in_capital',
    '私募股數': 'private_shares_num',
    '特別股': 'special_shares_num',
    '編制財務報表類型': 'financial_report_type',
    '股票過戶機構': 'stock_transfer_agency',
    '過戶電話': 'stcok_transfer_phone',
    '過戶地址': 'stock_transfer_address',
    '簽證會計師事務所': 'accounting_firm',
    '簽證會計師1': 'accountant_1',
    '簽證會計師2': 'accountant_2',
    '英文簡稱': 'stock_name_en',
    '英文通訊地址': 'address_en',
    '傳真機號碼': 'fax',
    '電子郵件信箱': 'email',
    '網址': 'website'
}

def convert_col_names(items:list):
    col_names = {}
    for name,value in template.items():
        if name in items:
            col_names[name]=value
    return col_names


import time
import datetime
import copy
from .utils import convert_num
from .tw_col_names import convert_table


fields = [
        "證券代號",
        "證券名稱",
        "外陸資買進股數(不含外資自營商)",
        "外陸資賣出股數(不含外資自營商)",
        "外陸資買賣超股數(不含外資自營商)",
        "外資自營商買進股數",
        "外資自營商賣出股數",
        "外資自營商買賣超股數",
        "投信買進股數",
        "投信賣出股數",
        "投信買賣超股數",
        "自營商買賣超股數",
        "自營商買進股數(自行買賣)",
        "自營商賣出股數(自行買賣)",
        "自營商買賣超股數(自行買賣)",
        "自營商買進股數(避險)",
        "自營商賣出股數(避險)",
        "自營商買賣超股數(避險)",
        "三大法人買賣超股數"
    ]
num_fields = [
        "外陸資買進股數(不含外資自營商)",
        "外陸資賣出股數(不含外資自營商)",
        "外陸資買賣超股數(不含外資自營商)",
        "外資自營商買進股數",
        "外資自營商賣出股數",
        "外資自營商買賣超股數",
        "投信買進股數",
        "投信賣出股數",
        "投信買賣超股數",
        "自營商買賣超股數",
        "自營商買進股數(自行買賣)",
        "自營商賣出股數(自行買賣)",
        "自營商買賣超股數(自行買賣)",
        "自營商買進股數(避險)",
        "自營商賣出股數(避險)",
        "自營商買賣超股數(避險)",
        "三大法人買賣超股數"
]


def parse(data,**kwargs):
    """
    資料來源:
        https://www.twse.com.tw/zh/page/trading/fund/T86.html
        https://www.twse.com.tw/fund/T86?response=json&date=20221012&selectType=ALL&_=1665588603850
    三大法人買賣超日報:
        key: data

    法人英文縮寫:
        法人:institutional investor => II
        外資:foreign investors => IIFI
        自營商:dealer => IID
        投信:investment trust => IIIT

    欄位依序為:
        "證券代號":stock_id
        "證券名稱":stock_name
        "外陸資買進股數(不含外資自營商)":IIFI_buy_amount_woIIFD
        "外陸資賣出股數(不含外資自營商)":IIFI_sell_amount_woIIFD
        "外陸資買賣超股數(不含外資自營商)":IIFI_net_amount_woIIFD
        "外資自營商買進股數":IIFD_buy_amount
        "外資自營商賣出股數":IIFD_sell_amount
        "外資自營商買賣超股數":IIFD_net_amount
        "投信買進股數":IIIT_buy_amount
        "投信賣出股數":IIIT_sell_amount
        "投信買賣超股數":IIIT_net_amount
        "自營商買賣超股數":IID_net_amount
        "自營商買進股數(自行買賣)":IID_buy_amount_self
        "自營商賣出股數(自行買賣)":IID_sell_amount_self
        "自營商買賣超股數(自行買賣)":IID_net_amount_self
        "自營商買進股數(避險)":IID_buy_amount_hedging
        "自營商賣出股數(避險)":IID_sell_amount_hedging
        "自營商買賣超股數(避險)":IID_net_amount_hedging
        "三大法人買賣超股數":II_net_amount
        
    資料範例第一筆(20221012):
        ['00632R',
        '元大台灣50反1   ',
        '9,475,000',
        '10,531,000',
        '-1,056,000',
        '0',
        '0',
        '0',
        '0',
        '0',
        '0',
        '33,113,360',
        '3,589,000',
        '2,837,000',
        '752,000',
        '78,438,681',
        '46,077,321',
        '32,361,360',
        '32,057,360']
    
    """


    formated_data = []

    status = data.get('stat')
    if status=='OK':
        if 'data' in data:
            rows = data['data']
            col_names = data['fields']
        else:
            print(f"Can't not find key 'data' in response data ")
            return formated_data

        for row in rows:
            # original data
            template = {}
            for col_name,value in zip(col_names,row):
                template[col_name]=value
            
            formated_row = {}
            #convert data
            for col_name,value in template.items():
                converted_col_name = convert_table[col_name]
                if col_name in num_fields:
                    value = convert_num(value)
                if col_name == '證券名稱':
                    value = value.strip()
                formated_row[converted_col_name]=value
            formated_row['date']=kwargs['date']
            formated_data.append(formated_row)
        
        return formated_data
    else:
        print(status)
        return formated_data

def gen_params(**params):
    """
    Require variable:
        date:20221012
    """

    parameters_template = {
        'fetch':{
            'url_template':'https://www.twse.com.tw/fund/T86?response=json&date=**date**&selectType=ALL&_=**time_stamp**',
            'url_params':{
                '**date**':'20221012',
                '**time_stamp**':str(int(time.time())),
            },
            'method':'GET',
            'response_type':'json'
        },
        'parse':{
            'parse_data':parse,
            'kwargs':{
                'date':'20221012',
            }
        }
    }

    parameters = copy.deepcopy(parameters_template)
    date_str = params['date']
    #format and verify string
    date_str = date_str.replace('-','')
    datetime.datetime.strptime(date_str,'%Y%m%d')
    
    parameters['fetch']['url_params']['**date**']=date_str
    parameters['parse']['kwargs']['date']=date_str

    return parameters

def gen_params_example():
    example_params = {'date':'20221012'}
    print('Get 3 institutional investors daily record!')
    print(f'ex:{example_params}')
    return example_params



import unittest
from unittest.mock import MagicMock, patch

from fin_crawler.fetcher import Fetcher


class TestFetcher(unittest.TestCase):

    @patch('fin_crawler.fetcher.requests')
    def test_text_response(self,mock_requests):

        mock_response = MagicMock()
        mock_response.status_code = 200
        mock_response.text = '123'

        mock_requests.get.return_value = mock_response

        fetch_params = {
            'url':'http://test.com/test',
            'method':'GET',
            'response_type':'text'
            }
        fetcher = Fetcher(fetch_params=fetch_params)
        self.assertEqual(fetcher.accept_codes,[200])
        self.assertEqual(fetcher.method,'get')
        self.assertEqual(fetcher.headers,None)
        self.assertEqual(fetcher.data,None)
        self.assertEqual(fetcher.response_type,'text')
        data = fetcher.fetch()

        self.assertEqual(data,'123')

    @patch('fin_crawler.fetcher.requests')
    def test_error500(self,mock_requests):

        mock_response = MagicMock()
        mock_response.status_code = 500

        mock_requests.get.return_value = mock_response

        fetch_params = {
            'url':'http://test.com/test',
            'method':'GET',
            'response_type':'text'
            }
        fetcher = Fetcher(fetch_params=fetch_params)
        with self.assertRaises(ConnectionError):
            fetcher.fetch()

    @patch('fin_crawler.fetcher.requests')
    def test_json_response(self,mock_requests):
        mock_response = MagicMock()
        mock_response.status_code = 200
        mock_response.json.return_value = {
            'test':'123'
        }
        mock_requests.get.return_value = mock_response
        fetch_params = {
            'url':'http://test.com/test',
            'method':'GET',
            }
        fetcher = Fetcher(fetch_params=fetch_params)
        self.assertEqual(fetcher.response_type,'json')
        data = fetcher.fetch()
        self.assertEqual(data,{'test':'123'})


    @patch('fin_crawler.fetcher.requests')
    def test_content_response(self,mock_requests):
        mock_response = MagicMock()
        mock_response.status_code = 200
        mock_response.content = b'123'
        mock_requests.get.return_value = mock_response
        fetch_params = {
            'url':'http://test.com/test',
            'method':'GET',
            'response_type':'content'
            }
        fetcher = Fetcher(fetch_params=fetch_params)
        self.assertEqual(fetcher.response_type,'content')
        data = fetcher.fetch()
        self.assertEqual(data,b'123')

    



import time
import datetime
import copy
from .utils import convert_num
from .tw_col_names import convert_table



fields = [
    "證券代號",
    "證券名稱",
    "成交股數",
    "成交筆數",
    "成交金額",
    "開盤價",
    "最高價",
    "最低價",
    "收盤價",
    "漲跌(+/-)",
    "漲跌價差",
    "最後揭示買價",
    "最後揭示買量",
    "最後揭示賣價",
    "最後揭示賣量",
    "本益比"
    ]
num_fields = [
    "成交股數",
    "成交筆數",
    "成交金額",
    "開盤價",
    "最高價",
    "最低價",
    "收盤價",
    "最後揭示買價",
    "最後揭示買量",
    "最後揭示賣價",
    "最後揭示賣量",
    "本益比"
]


def parse(data,**kwargs):

    """
    資料來源:
        https://www.twse.com.tw/zh/page/trading/exchange/MI_INDEX.html
        https://www.twse.com.tw/exchangeReport/MI_INDEX?response=json&date=20220920&type=ALL&_=1663693519652
    當日股價:
        key: data9 (新版)
        key: data8 (舊版)
    欄位依序為:
        "證券代號":'stock_id',
        "證券名稱":'stock_name',
        "成交股數":'volume',
        "成交筆數":'trade_num',
        "成交金額":'trade_amount',
        "開盤價":'open',
        "最高價":'high',
        "最低價":'low',
        "收盤價":'close',
        "漲跌(+/-)":'up_down',
        "漲跌價差":'spread_value',
        "最後揭示買價":'last_buy_price',
        "最後揭示買量":'last_buy_volume',
        "最後揭示賣價":'last_sell_price',
        "最後揭示賣量":'last_sell_volume',
        "本益比":'PE',
        '日期': 'date',
    資料範例(20220920第一筆(0050)):
        ['0050',
        '元大台灣50',
        '5,999,746',
        '7,093',
        '675,492,164',
        '112.55',
        '113.15',
        '112.25',
        '113.05',
        '<p style= color:red>+</p>',
        '1.00',
        '113.00',
        '205',
        '113.05',
        '9',
        '0.00']
    
    return data:
        formated_data = [
            {
                'stock_id':'2330',
                'stock_name':'台積電',
                'volume':xxx,
                'trade_amount':xxx,
                'open':xxx,
                'high':xxx,
                'low':xxx,
                'spread':xxx
                'date','2022-10-14'
            }
        ]

    """

    
    formated_data = []

    status = data.get('stat')
    if status=='OK':
        if 'data9' in data:
            rows = data['data9']
            col_names = data['fields9']
        elif 'data8' in data:
            rows = data['data8']
            col_names = data['fields8']
        else:
            return formated_data

        for row in rows:
            # original format
            template = {}
            for col_name,value in zip(col_names,row):
                template[col_name]=value

            formated_row = {}
            # format data
            for col_name,value in template.items():
                convert_col_name = convert_table[col_name]

                # convert to number and insert
                if col_name in num_fields:
                    value = convert_num(value)
                if col_name not in ['漲跌(+/-)','漲跌價差']:
                    formated_row[convert_col_name]=value
            
            # add converted data
            formated_row['spread'] = convert_num(template['漲跌價差'])*1 if '+' in template['漲跌(+/-)'] else -1
            # add date
            formated_row['date'] = kwargs.get('date') or ''
            # append into array
            formated_data.append(formated_row)

        return formated_data
    else:
        print(status)
        return formated_data




def gen_params(**params):
    """
    Require Variable: 
        date:20220202

    fetch:
        url_template:'https://www.twse.com.tw/exchangeReport/MI_INDEX?response=json&date=**date**&type=ALL&_=**time_stamp**'
        url_parms:
            **date**:date (ex:20040925)
            **time_stamp**: current time stamp befor '.' (ex:1663778386)
        method:GET
        response_type:json
    parse:
        parse_data:parse_stock_tw_daily
    """

    parameters_template = {
        'fetch':{
            'url_template':'https://www.twse.com.tw/exchangeReport/MI_INDEX?response=json&date=**date**&type=ALL&_=**time_stamp**',
            'url_params':{
                '**date**':'20040925',
                '**time_stamp**':str(int(time.time()))
            },
            'method':'GET',
            'response_type':'json'
        },
        'parse':{
            'parse_data':parse,
            'kwargs':{
                'date':'20040925'
            }
        }
    }

    parameters = copy.deepcopy(parameters_template)
    date_str = params['date']
    #format and verify string
    date_str = date_str.replace('-','')
    datetime.datetime.strptime(date_str,'%Y%m%d')
    
    parameters['fetch']['url_params']['**date**']=date_str
    parameters['parse']['kwargs']['date']=date_str
    return parameters

def gen_params_example():
    example_params = {'date':'20220920'}
    print('Get daily stock price!')
    print(f'ex:{example_params}')
    return example_params





import unittest
from fin_crawler import FinCrawler
import time
class Test_tw_stock_info_real(unittest.TestCase):

    def test_request(self):
        time.sleep(4)
        result_idx1 = {
            'update_date': '20221015',
            'stock_id': '1102',
            'company_name': '亞洲水泥股份有限公司',
            'stock_name': '亞泥',
            'foreign_register_country': '－ ',
            'industry_type': '01',
            'address': '台北市大安區敦化南路2段207號30、31樓',
            'tax_id': '03244509',
            'chairman': '徐旭東',
            'CEO': '李坤炎',
            'spokesman': '周維崑',
            'spokesman_title': '副總經理',
            'deputy_spokesman': '吳玲綾',
            'phone': '02-2733-8000',
            'establishment_date': '19570321',
            'IPO_date': '19620608',
            'common_shares_price': '新台幣                 10.0000元',
            'paid_in_capital': '35459275570',
            'private_shares_num': '0',
            'special_shares_num': '0',
            'financial_report_type': '1',
            'stock_transfer_agency': '亞東證券股份有限公司',
            'stcok_transfer_phone': '02-7753-1699',
            'stock_transfer_address': '新北市板橋區新站路16號13樓',
            'accounting_firm': '勤業眾信聯合會計師事務所',
            'accountant_1': '戴信維',
            'accountant_2': '陳培德',
            'stock_name_en': 'ACC',
            'address_en': "30-31F., No.207, Sec. 2, Dunhua S. Rd., Da' an Dist., Taipei City 106, TaiwanTAIPEI,TAIWAN,R.O.C",
            'fax': '02-2378-5191',
            'email': 'service@acc.com.tw',
            'website': 'www.acc.com.tw'
        }
        result_idx500 = {
            'update_date': '20221015',
            'stock_id': '2923',
            'company_name': '鼎固控股有限公司',
            'stock_name': '鼎固-KY',
            'foreign_register_country': 'KY 開曼群島                                                    ',
            'industry_type': '14',
            'address': '臺北市信義區基隆路一段420號10樓(臺北聯絡處)',
            'tax_id': '31896684',
            'chairman': '張洪本',
            'CEO': '張能耀',
            'spokesman': '張能耀',
            'spokesman_title': '總經理',
            'deputy_spokesman': '馮安怡',
            'phone': '8621-64332999',
            'establishment_date': '20071205',
            'IPO_date': '20121207',
            'common_shares_price': '新台幣                 10.0000元',
            'paid_in_capital': '17402969820',
            'private_shares_num': '0',
            'special_shares_num': '0',
            'financial_report_type': '1',
            'stock_transfer_agency': '富邦綜合證券(股)公司股務代理部',
            'stcok_transfer_phone': '(02)23611300',
            'stock_transfer_address': '台北市許昌街17號2樓',
            'accounting_firm': '勤業眾信聯合會計師事務所',
            'accountant_1': '鄭旭然',
            'accountant_2': '陳俊宏',
            'stock_name_en': 'Sino Horizon',
            'address_en': '450#, Ruijin 2 road, Huangpu District,Shanghai, PRC',
            'fax': '8621-64332999',
            'email': 'ir@sinohorizon.cn',
            'website': 'http://www.sinohorizon.cn/'
            }
        data = FinCrawler.get('tw_stock_info',{})
        self.assertEqual(data[1],result_idx1)
        self.assertEqual(data[500],result_idx500)





import requests

class Fetcher:
    """
    Handling requests action including sending request and parse response object

    fetch_params:
        url: requests url
        method: requests method
        headers: requests headers
        data: data send with requests
        response_type: response parsing type (json/text/content)
        accept_codes: accepted response code, code usually will be 200 if not will raise ConnectionError
    """

    def __init__(self,fetch_params):
        self.url = fetch_params['url']
        self.method = fetch_params['method'].lower()
        self.headers = fetch_params.get('headers')
        self.data = fetch_params.get('data')
        self.response_type = fetch_params.get('response_type') or 'json'
        self.accept_codes = fetch_params.get('accept_codes') or [200]
    def fetch(self):
        response = eval(f"self.{self.method}()")
        if response.status_code in self.accept_codes:
            response_data = self.parse_response(response)
            return response_data
        else:
            raise ConnectionError(f"Response Error:{response.content}")
    def get(self):
        return requests.get(self.url,headers=self.headers)
    
    def parse_response(self,response):
        if self.response_type=='json':
            return response.json()
        elif self.response_type=='text':
            return response.text
        else:
            return response.content